/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;

/**
 * 上传文件检查规则
 *
 * @author RenFengJiang
 * @date 2024/1/8
 */

@Rule(key = "UploadFileVerifyChecker")
public class UploadFileVerifyChecker extends IssuableSubscriptionVisitor {

    private final String value = "程序设计时，应以“白名单”方式限制允许用户上传的文件的类型";
    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.METHOD);
    }

    @Override
    public void visitNode(Tree tree) {
        MethodTree node = (MethodTree) tree;
        List<VariableTree> parameters = node.parameters();
        //盘带是否是文件上传类
        boolean boo = parameters.stream().anyMatch(type -> "MultipartFile".equals(type.type().toString()));
        if (boo) {
            //获取文件名称类型判断是否配置文件权限
            var interiorInvoIf = new InteriorInvoIf();
            ((MethodTree) tree).block().accept(interiorInvoIf);
            if (!interiorInvoIf.fileType.equals("")) {
                //判断是否对文件后缀进行限制
                NodeIf nodeIf = new NodeIf(interiorInvoIf.fileType);
                ((MethodTree) tree).block().accept(nodeIf);
                if (nodeIf.boo) {
                    context.reportIssue(this, node.simpleName(), value);
                }
            } else {
                if (!interiorInvoIf.fileName.equals("")) {
                    //                判断是否对文件后缀进行限制
                    NodeIf nodeIf = new NodeIf(interiorInvoIf.fileName);
                    ((MethodTree) tree).block().accept(nodeIf);
                    if (nodeIf.boo) {
                        context.reportIssue(this, node.simpleName(), value);
                    }
                } else {
                    context.reportIssue(this, node.simpleName(), value);
                }
            }

            if (!interiorInvoIf.sizeName.equals("")) {
                //判断是否对文件大小进行限制
                NodeIf nodeIf = new NodeIf(interiorInvoIf.sizeName);
                //nodeIf.check(((MethodTree) tree).block());
                ((MethodTree) tree).block().accept(nodeIf);
                if (nodeIf.boo) {
                    context.reportIssue(this, node.simpleName(), value);
                }
            }
            //判断是否进行权限设置
            if (interiorInvoIf.privType) {
                context.reportIssue(this, node.simpleName(), value);
            }
        }

    }

    //內部文件名称类型获取类
    private class InteriorInvoIf extends BaseTreeVisitor {
        //文件全名字
        public String fileName = "";
        //    文件后缀名
        public String fileType = "";
        //文件大小
        public String sizeName = "";
        //判断权限
        public boolean privType = true;

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            //获取到方法调用的参数
            ExpressionTree expressionTree = tree.methodSelect();
            if (expressionTree instanceof MemberSelectExpressionTree) {
                MemberSelectExpressionTree expressionTree1 = (MemberSelectExpressionTree) expressionTree;
                //对调用方法进行判断
                if ("getOriginalFilename".equals(expressionTree1.identifier().toString())) {
                    Tree parent = expressionTree1.parent();
                    if (parent instanceof MethodInvocationTree) {
                        MethodInvocationTree memberSelectExpressionTree = (MethodInvocationTree) parent;
                        Tree parent1 = memberSelectExpressionTree.parent();
                        if (parent1 instanceof VariableTree) {
                            VariableTree variableTree = (VariableTree) parent1;
                            fileName = variableTree.simpleName().toString();
                        }
                    }
                } else if ("extName".equals(expressionTree1.identifier().toString())) {
                    Tree parent = expressionTree1.parent();
                    if (parent instanceof MethodInvocationTree) {
                        MethodInvocationTree memberSelectExpressionTree = (MethodInvocationTree) parent;
                        Tree parent1 = memberSelectExpressionTree.parent();
                        if (parent1 instanceof VariableTree) {
                            VariableTree variableTree = (VariableTree) parent1;
                            fileType = variableTree.simpleName().toString();
                        }
                    }
                } else if ("getSize".equals(expressionTree1.identifier().toString())) {
                    Tree parent = expressionTree1.parent();
                    if (parent instanceof MethodInvocationTree) {
                        MethodInvocationTree memberSelectExpressionTree = (MethodInvocationTree) parent;
                        Tree parent1 = memberSelectExpressionTree.parent();
                        if (parent1 instanceof VariableTree) {
                            VariableTree variableTree = (VariableTree) parent1;
                            sizeName = variableTree.simpleName().toString();
                        }
                    }
                } else if ("setExecutable".equals(expressionTree1.identifier().toString()) || "setReadable".equals(expressionTree1.identifier().toString()) || "setWritable".equals(expressionTree1.identifier().toString())) {
                    privType = false;
                }
            }
        }
    }


    public class NodeIf extends BaseTreeVisitor {

        private String name;
        public boolean boo = true;

        public NodeIf(String name) {
            this.name = name;
        }

        @Override
        public void visitIfStatement(IfStatementTree tree) {
            //获取到if表达式
            ExpressionTree condition = tree.condition();
            if (condition instanceof BinaryExpressionTree) {
                BinaryExpressionTree binaryExpressionTree = (BinaryExpressionTree) condition;
                //判断是否进行if判断
                if (name.equals(binaryExpressionTree.leftOperand().toString())) {
                    boo = false;
                } else if (name.equals(binaryExpressionTree.rightOperand().toString())) {
                    boo = false;
                }
            }
        }
    }
}
